# Copyright (C) 2012  Stian Ellingsen <stian@plaimi.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import pygame


class SurfaceScaler(object):

    """Provides integer upscaling from one surface to another"""

    def __init__(self, scale=1, dest=None):
        self._scale = scale
        self._dest = dest
        self._source = None

    @property
    def scale(self):
        """The scaling factor"""
        return self._scale

    @scale.setter
    def scale(self, scale):
        self._scale = max(1, int(scale))
        self._source = None

    @property
    def dest(self):
        """The destination surface for the scaled image"""
        return self._dest

    @dest.setter
    def dest(self, dest):
        self._dest = dest
        self._source = None

    @property
    def source(self):
        """The source surface with the image to be scaled"""
        if self._source is None:
            dest, scale = self._dest, self._scale
            if scale == 1:
                self._source = dest
            elif dest is not None:
                size, flags = dest.get_size(), dest.get_flags()
                self._source = pygame.Surface(
                        (size[0] // scale, size[1] // scale),
                        flags & 0x0FFFFFFF, dest)
        return self._source

    @staticmethod
    def _scale(source, dest):
        pygame.transform.scale(source, dest.get_size(), dest)

    def update(self, rectangles=None):
        """Update the destination with all or parts of the source surface"""
        scale = self._scale
        if scale == 1:
            return rectangles
        if rectangles is None:
            SurfaceScaler._scale(self._source, self._dest)
            return
        drs = [pygame.Rect(r.x * scale, r.y * scale, r.w * scale, r.h * scale)
              for r in rectangles]
        for sr, dr in zip(rectangles, drs):
            SurfaceScaler._scale(self.source.subsurface(sr),
                                 self.dest.subsurface(dr))
        return drs
