# Copyright (C) 2012  Stian Ellingsen <stian@plaimi.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


class BitmapCharacter(object):

    def __init__(self, surface, rect, bearing, advance):
        self.surface = surface
        self.rect = rect
        self.bearing = bearing
        self.advance = advance


class BitmapFont(object):

    def __init__(self, height, ascent, descent):
        self._characters = {}
        self._kerning = {}
        self.height = height
        self.ascent = ascent
        self.descent = descent

    @staticmethod
    def character_index(c):
        return c if isinstance(c, int) else ord(c)

    @staticmethod
    def kerning_index(pair):
        return pair if isinstance(pair, unicode) else unicode(pair, 'utf-8')

    def get_character(self, c):
        return self._characters.get(BitmapFont.character_index(c))

    def get_kerning(self, pair):
        return self._kerning.get(BitmapFont.kerning_index(pair), 0)

    def set_character(self, c, bc):
        i = BitmapFont.character_index(c)
        if bc is None:
            self._characters.pop(i, None)
        else:
            self._characters[i] = bc

    def set_kerning(self, pair, k):
        i = BitmapFont.kerning_index(pair)
        if k == 0:
            self._kerning.pop(i, None)
        else:
            self._kerning[i] = k


class BitmapString(object):

    def __init__(self, font, text="", spacing=0):
        self._font = font
        self._text = text
        self._spacing = 0
        self._width = -1

    @property
    def width(self):
        if self._width < 0:
            f, t = self._font, self._text
            w = sum([f.get_character(c).advance for c in t]) + \
                sum([f.get_kerning(a + b) for a, b in zip(t[:-1], t[1:])])
            self._width = w
        return self._width

    def try_append(self, t, maxwidth):
        w = self.width
        f = self._font
        w += sum([f.get_character(c).advance for c in t]) + \
             sum([f.get_kerning(a + b) for a, b in zip(t[:-1], t[1:])])
        if len(self._text) > 0:
            w += f.get_kerning(self._text[-1] + t[0])
        if w <= maxwidth:
            self._width = w
            self._text += t
            return True
        return False

    def render(self, surface, pos):
        f = self._font
        x, y = pos
        r = []
        for c in self._text:
            ch = f.get_character(c)
            if (ch.surface is not None) and (ch.rect is not None):
                bx, by = ch.bearing
                cr = surface.blit(ch.surface, (x + bx, y - by), ch.rect)
                if r.width > 0 and r.height > 0:
                    r += [cr]
            x += ch.advance
        return r
