"""
'Character' can be a player character or a non-player character. It has a
'die' that it uses to perform most actions.

It has three attributes. 'cleverness', 'ment_might' and 'phys_might'. These
dictate how smart the character is, how good it is performing mental tasks
(magic is considered a mental task) and how good it is at performing physical
tasks.

There are six skills.  'defence' is used when being attacked.  'melee' is used
when attacking with a weapon that can only affect adjacent tiles.  'perception'
is used for finding hidden (stealthing) characters.  'ranged' is used when
attacking with a weapon that can affect non-adjacent tiles.  'speech' is used
for all dialogue.  'stealth' is used for hiding.  'stealth' cannot be used with
'heavy armour'.

A 'character' has two pools.  One 'health' pool, and one 'magic' pool.  They are
the points the character has to spend.  Being attacked decrements 'health'.  
Using magic decrements 'magic'. 

'armour' is the total armour score, calculated from all 'Armour' 'Item's worn
by the 'Character'.  'armour_class' is the highest 'armour_class' currently
equipped.  'weapon' is the currently equipped weapon.

'hit()' is used to attack something, and 'defend()' is used to defend against
something. 'damage(crit)' is used to damage something, and 'pain(damage)' is
used to decrement 'health'.  The 'crit' parameter in 'damage(crit)' is whether
it was a critical hit or not.  (See "weapon.py".)

'equip(item)' is used to equip an item to its appropriate slot, and
'unequip(item)' is used to unequip an item from its slot, and put the 'empty'
item in its place.  (See "items.py".)  'slots{}' contains all the equipable
slots.  It is initialised with the 'empty' item.
"""
from nei.armour import Armour
from nei.dice import roll_die
from nei.items import empty
from nei.weapon import Weapon


class Character():

    def __init__(self, cleverness, ment_might, phys_might, defence, melee,
                 perception, ranged, speech, stealth, health, magic):
        # Attributes
        self.cleverness = cleverness
        self.ment_might = ment_might
        self.phys_might = phys_might

        # Skills
        self.defence = defence
        self.melee = melee
        self.perception = perception
        self.ranged = ranged
        self.speech = speech
        self.stealth = stealth

        # Pools
        self.health = health
        self.magic = magic

        # Character die.  Used for all character rolls.
        self.die = 20

        # All the slots where items may be equipped.
        self.slots = {'big_container': empty, 'small_container': empty,
                      'boots': empty, 'belt': empty, 'left_ring': empty,
                      'left_hand': empty, 'right_hand': empty, 'hands': empty,
                      'wrists': empty, 'body': empty, 'back': empty,
                      'neck': empty, 'head': empty, 'end': empty}

        # The score of all equipped armour items.
        self.armour = 0
        # The highest armour class of all items.
        self.armour_class = 0

    def damage(self, crit):
        return self.phys_might + self.weapon.damage(crit)

    def defend(self):
        # Defend against a hit with the armour score + the defence skill.
        return self.armour + self.defence

    def equip(self, item):
        # Unequip what is currently equipped to the slot
        self.unequip(item.slot)
        # If equipping double-handed item, unequip one-handed items.
        if (item.slot is 'hands'):
            self.unequip('left_hand')
            self.unequip('right_hand')
        # If equipping one-handed item, unequip double-handed item.
        elif (item.slot is 'right_hand') or (item.slot is 'left_hand'):
            self.unequip('hands')

        # Update the weapon if it's a weapon
        if isinstance(item, Weapon):
            self.weapon = item
        # Update the armour score and armour class if it's an armour
        elif isinstance(item, Armour):
            self.armour = self.armour + item.defence
            self.armour_class = max(self.armour_class, item.armour_class)

        # Actually equip the new item.
        self.slots[item.slot] = item
    
    def hit(self):
        # Roll the die
        roll = roll_die(self.die)
        # Return 0 if the roll is above or equal to the die size minus the
        # critical strike range of the weapon.  The 0 means that the character
        # has performed a critical hit.  A critical hit always hits what it is
        # trying to.
        #
        # If there is no critical hit, check the range of the weapon.  If it
        # is a melee weapon, use the melee skill, else use the ranged skill.
        # Add the appropriate skill to the roll and return it.
        return 0 if roll >= (self.die - self.weapon.crit_range) else roll + \
                 (self.melee if (self.weapon.range_ is 1) else self.ranged)

    def pain(self, damage):
        # Reduce the health pool with the pain dealt.
        self.health = self.health - damage

    def unequip(self, slot):
        # Unequip the active weapon if it's a weapon
        if isinstance(self.slots[slot], Weapon):
            self.weapon = empty
        # Reduce the armour score if it's an armour
        if isinstance(self.slots[slot], Armour):
            self.armour = self.armour - self.slots[slot].defence
            self.armour_class = 0
            for i in self.slots:
                if isinstance(i, Armour):
                    self.armour_class = max(i, self.armour_class)

        # Equip the empty item to a slot.
        self.slots[slot] = empty
